@@ -190,7 +190,12 @@ class Request(Message):
     def bind_user_connection(self, lo):
         CORE.process('LDAP bind for user %r.' % (self.user_dn,))
         try:
-            if self.auth_type == 'SAML':
+            if self.auth_type == 'OIDC':
+                lo.lo.bind_oauthbearer(None, self.password)
+                if not lo.lo.compare_dn(lo.binddn, self.user_dn):
+                    CORE.warn('OIDC binddn does not match: %r != %r' % (lo.binddn, self.user_dn))
+                    self.user_dn = lo.binddn
+            elif self.auth_type == 'SAML':
                 lo.lo.bind_saml(self.password)
                 if not lo.lo.compare_dn(lo.binddn, self.user_dn):
                     CORE.warn('SAML binddn does not match: %r != %r' % (lo.binddn, self.user_dn))
