@@ -384,7 +384,10 @@ class Logout(Resource):
     ignore_session_timeout_reset = True
 
     def get(self, **kwargs):
-        if self.current_user.saml is not None:
+        session = self.current_user
+        if session.oidc is not None:
+            return self.redirect('/univention/oidc/logout', status=303)
+        if session.saml is not None:
             return self.redirect('/univention/saml/logout', status=303)
 
         self.expire_session()
@@ -961,7 +964,7 @@ class Set(Resource):
             if is_univention_lib and cls:
                 # for backwards compatibility with non redirecting clients we cannot redirect here :-(
                 p = cls(self.application, self.request)
-                p._ = self._
+                p.locale = self.locale
                 p.finish = self.finish
                 await p.post()
                 return
