@@ -86,7 +86,7 @@ class User(object):
 class Session(object):
     """A interface to session data"""
 
-    __slots__ = ('session_id', 'acls', 'user', 'saml', 'processes', '_timeout_id', '_active_requests', '_')
+    __slots__ = ('session_id', 'acls', 'user', 'saml', 'oidc', 'processes', '_timeout_id', '_active_requests', '_')
     __auth = AuthHandler()
     sessions = {}
 
@@ -116,6 +116,7 @@ class Session(object):
         self.session_id = session_id
         self.user = User()
         self.saml = None
+        self.oidc = None
         self.acls = IACLs(self)
         self.processes = Processes(self)
         self._timeout_id = None
@@ -212,14 +213,25 @@ class Session(object):
         # though
         return self.user.password is None and self.saml
 
+    def is_oidc_user(self):
+        # self.oidc indicates that it was originally a
+        # oidc user. but it may have upgraded and got a
+        # real password. the oidc user object is still there,
+        # though
+        return self.user.password is None and self.oidc
+
     def get_umc_password(self):
+        if self.is_oidc_user():
+            return self.oidc.access_token
         if self.is_saml_user():
             return self.saml.message
         else:
             return self.user.password
 
     def get_umc_auth_type(self):
-        if self.is_saml_user():
+        if self.is_oidc_user():
+            return "OIDC"
+        elif self.is_saml_user():
             return "SAML"
         else:
             return None
@@ -255,11 +267,15 @@ class Session(object):
             ioloop = tornado.ioloop.IOLoop.current()
             ioloop.remove_timeout(self._timeout_id)
 
-    def timed_out(self, now):
-        return self.session_end_time < now
+    def timed_out(self, now=None, session_end_time=None):
+        now = now if now else monotonic()
+        session_end_time = session_end_time if session_end_time else self.session_end_time
+        return session_end_time < now
 
     @property
     def session_end_time(self):
+        if self.is_oidc_user() and self.oidc.session_end_time:
+            return self.oidc.session_end_time
         if self.is_saml_user() and self.saml.session_end_time:
             return self.saml.session_end_time
         return self.user.session_end_time
